const fs = require('fs')
const path = require('path')
const shell = require('shelljs');
const colors = require('colors');
const store = require('../store/index.js');

/**
 * 获取输入参数列表
 */
function getInputs() {
    process.argv.splice(0, 2)
    return process.argv
}

/**
 * 项目建立路径
 */
function getProjectNameFromInput() {
    let dir = getInputs()[0] || ''
    return store.CWD_PATH + '/' + dir
}

/**
 * 路径是否存在
 * @param {String} path 
 */
function fsExistsSync(path) {
    try {
        fs.accessSync(path, fs.F_OK)
    } catch (e) {
        return false
    }
    return true
}

/**
 * 拷贝文件
 * @param {String} _targetPath 目标
 * @param {String} _templatePath 源
 */
function copyFile(_targetPath, _templatePath) {
    fs.writeFileSync(_targetPath, fs.readFileSync(_templatePath), "utf-8")
}

/**
 * 拷贝文件夹
 * @param {String} templatePath
 * @param {String} targetPath
 */
function copyDir(templatePath, targetPath) {
    try {
        if (fs.statSync(templatePath).isDirectory() && !fsExistsSync(targetPath)) {
            fs.mkdirSync(targetPath)
        }
        const paths = fs.readdirSync(templatePath)
        paths.forEach(_path => {
            const _targetPath = path.resolve(targetPath, _path)
            const _templatePath = path.resolve(templatePath, _path)
            console.log("creating..." + _targetPath)
            if (!fs.statSync(_templatePath).isFile()) {
                fs.mkdirSync(_targetPath)
                copyDir(_templatePath, _targetPath)
            } else {
                copyFile(_targetPath, _templatePath)
            }
        })
    } catch (error) {
        console.log(error)
        return false
    }
    return true
}

/**
 * 安装依赖
 * @param {String} projectName 
 */
function installDependencies(projectName) {
    console.log('\ninstalling dependencies...');
    if (shell.exec(`cd ${projectName} && npm install --registry=http://registry.cnpmjs.org`).code !== 0) {
        if (shell.exec(`cd ${projectName} && npm install`).code !== 0) {
            console.log(colors.red('install dependencies failed!'));
            return false
        } else {
            return true
        }
    } else {
        return true
    }
}

/**
 * 新建项目
 */
function newProject() {
    const projectName = getProjectNameFromInput()
    let isOk
    isOk = copyDir(store.TPL_PATH, projectName)
    if (isOk) {
        isOk = installDependencies(projectName)
    }

    if (isOk) {
        console.log(
            colors.yellow(
                [
                    `Success! Created ${projectName} at ${store.CWD_PATH}.`,
                    'Start by typing:',
                    `  cd ${projectName}`,
                    '  npm run dev',
                    'Happy hacking!'
                ].join('\n')
            )
        );
    }
}

module.exports = {
    getInputs,
    getProjectNameFromInput,
    copyFile,
    copyDir,
    installDependencies,
    newProject
}